<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: lib-install-db.inc.php 1235 2005-01-17 16:36:51Z andrew $
*/

// Define
define('phpAds_tableTypesSupported', true);

/*-------------------------------------------------------*/
/* Check if the database already exists                  */
/*-------------------------------------------------------*/

function phpAds_checkDatabaseExists()
{
    // Get the database structure
    $dbstructure 	 = phpAds_prepareDatabaseStructure();
    $availabletables = array();
    // Get table names
    $res = phpAds_dbQuery("SHOW TABLES");
    while ($row = phpAds_dbFetchRow($res)) {
        $availabletables[] = $row[0];
    }
    $result = false;
    for (reset($dbstructure); $key = key($dbstructure); next($dbstructure)) {
        if (is_array($availabletables) && in_array($key, $availabletables)) {
            // Table exists
            $result = true;
            break;
        }
    }
    return $result;
}

/*-------------------------------------------------------*/
/* Check if the database is valid                        */
/*-------------------------------------------------------*/

function phpAds_checkDatabaseValid()
{
    // Get the database structure
    $dbstructure = phpAds_prepareDatabaseStructure();
    $result      = true;
    // Get table names
    $res = phpAds_dbQuery("SHOW TABLES");
    while ($row = phpAds_dbFetchRow($res)) {
        $availabletables[] = $row[0];
    }
    for (reset($dbstructure); $key = key($dbstructure); next($dbstructure)) {
        if (is_array($availabletables) && in_array($key, $availabletables)) {
            // Table exists, check if it is valid
            if (!phpAds_checkTable($key, $dbstructure[$key])) {
                $result = false;
            }
        } else {
            $result = false;
        }
    }
    return ($result);
}

/*-------------------------------------------------------*/
/* Create the database                                   */
/*-------------------------------------------------------*/

function phpAds_createDatabase($tabletype = '')
{
    // Get the database structure
    $dbstructure = phpAds_prepareDatabaseStructure();
    $availabletables = array();
    // Get table names
    $res = phpAds_dbQuery("SHOW TABLES");
    while ($row = phpAds_dbFetchRow($res)) {
        $availabletables[] = $row[0];
    }
    for (reset($dbstructure); $key = key($dbstructure); next($dbstructure)) {
        if (is_array($availabletables) && in_array($key, $availabletables)) {
            // Table exists, drop it
            phpAds_dropTable($key);
        }
        // Table doesn't exists, create
        phpAds_createTable($key, $dbstructure[$key], $tabletype);
        if (preg_match('/application_variable/', $key)) {
            // Insert the Max version into the database
            global $phpAds_version_readable;
            $query = "INSERT INTO $key (name, value) VALUES ('max_version', '$phpAds_version_readable')";
            phpAds_dbQuery($query);
        }
    }
    return true;
}

/*-------------------------------------------------------*/
/* Check if a table has the correct structure            */
/*-------------------------------------------------------*/

function phpAds_checkTable($name, $structure)
{
    $result  = true;
    $columns = $structure['columns'];
    if (isset($structure['primary'])) $primary = $structure['primary'];
    if (isset($structure['index']))   $index   = $structure['index'];
    if (isset($structure['unique']))  $unique  = $structure['unique'];
    // Get existing columns
    $res = phpAds_dbQuery("DESCRIBE ".$name);
    while ($row = phpAds_dbFetchArray($res)) {
        $availablecolumns[$row['Field']] = $row;
    }
    // Check columns
    for (reset($columns); $key = key($columns); next($columns)) {
        if (!(isset($availablecolumns[$key]) && is_array($availablecolumns[$key]))) {
            $result = false;
        }
    }
    return $result;
}

/*-------------------------------------------------------*/
/* Create a table                                        */
/*-------------------------------------------------------*/

function phpAds_createTable($name, $structure, $tabletype = '')
{
    $columns = $structure['columns'];
    if (isset($structure['primary'])) $primary = $structure['primary'];
    if (isset($structure['index']))   $index   = $structure['index'];
    if (isset($structure['unique']))  $unique  = $structure['unique'];
    // Create empty array
    $createdefinitions = array();
    // Add columns
    for (reset($columns); $key = key($columns); next($columns)) {
        $createdefinitions[] = $key." ".$columns[$key];
    }
    if (isset($primary) && is_array($primary)) {
        $createdefinitions[] = "PRIMARY KEY (".implode(",", $primary).")";
    }
    if (isset($index) && is_array($index)) {
        for (reset($index); $key = key($index); next($index)) {
            $createdefinitions[] = "INDEX $key (".implode(",", $index[$key]).")";
        }
    }
    if (isset($unique) && is_array($unique)) {
        for (reset($unique); $key = key($unique); next($unique)) {
            $createdefinitions[] = "UNIQUE $key (".implode(",", $unique[$key]).")";
        }
    }
    if (is_array($createdefinitions) && sizeof($createdefinitions) > 0) {
        $query  = "CREATE TABLE $name (";
        $query .= implode (", ", $createdefinitions);
        $query .= ")";
        // Tabletype
        if ($tabletype != '') {
            $query .= " TYPE=".$tabletype;
        }
        phpAds_dbQuery($query);
    }
}

/*-------------------------------------------------------*/
/* Drop an existing table                                */
/*-------------------------------------------------------*/

function phpAds_dropTable($name)
{
    return phpAds_dbQuery("DROP TABLE ".$name);
}


/*-------------------------------------------------------*/
/* Get table types                                       */
/*-------------------------------------------------------*/

function phpAds_getTableTypes()
{
    $types['MYISAM'] = 'MyISAM';
    $types['BDB'] = 'Berkeley DB';
    $types['GEMINI'] = 'NuSphere Gemini';
    $types['INNODB'] = 'InnoDB';
    return $types;
}

function phpAds_checkTableType($type)
{
    // Assume MySQL always supports MyISAM table types
    if ($type == 'MYISAM') {
        return true;
    } else {
        $res = phpAds_dbQuery("SHOW VARIABLES");
        while ($row = phpAds_dbFetchRow($res)) {
            if ($type == 'BDB' && $row[0] == 'have_bdb' && $row[1] == 'YES') {
                return true;
            }
            if ($type == 'GEMINI' && $row[0] == 'have_gemini' && $row[1] == 'YES') {
                return true;
            }
            if ($type == 'INNODB' && $row[0] == 'have_innodb' && $row[1] == 'YES') {
                return true;
            }
        }
    }
    return false;
}

/*-------------------------------------------------------*/
/* Get the default table type                            */
/*-------------------------------------------------------*/

function phpAds_getTableTypeDefault()
{
    /*
    $res = phpAds_dbQuery("SHOW VARIABLES");
    while ($row = phpAds_dbFetchRow($res)) {
    if ($row[0] == 'table_type') {
    return $row[1];
    }
    }
    return false;
    */
    return 'MYISAM';
}

/*-------------------------------------------------------*/
/* Read the database structure from a sql file           */
/*-------------------------------------------------------*/

function phpAds_readDatabaseStructure()
{
    global $phpAds_config;
    $sql = join("", file(phpAds_path."/libraries/defaults/all.sql"));
    // Stripping comments
    $sql = ereg_replace("$-- [^\n]*\n", "\n", $sql);
    $sql = ereg_replace("$#[^\n]*\n", "\n", $sql);
    $sql = str_replace("\n", " ", $sql);
    $sql = str_replace("\r", " ", $sql);
    // Unifying duplicate blanks
    $sql = ereg_replace("[[:blank:]]+", " ", $sql);
    $sql = explode(";", $sql);
    // Replacing table names to match config.inc.php
    for ($i = 0; $i < sizeof($sql); $i++) {
        if (ereg("CREATE TABLE (max_[^\(]*) \(", $sql[$i], $regs)) {
            $tablename = str_replace("max_", "tbl_", $regs[1]);
            if (isset($phpAds_config[$tablename])) {
                $sql[$i] = str_replace($regs[1], $phpAds_config[$tablename], $sql[$i]);
            }
        }
    }
    // Create an array with an element for each query
    return $sql;
}

/*-------------------------------------------------------*/
/* Parse the an sql file and return all queries          */
/*-------------------------------------------------------*/

function phpAds_prepareDatabaseStructure()
{
    $dbstructure = array();
    // Read the all.sql file
    $createStatements = phpAds_readDatabaseStructure();
    foreach ($createStatements as $createStatement) {
        if (preg_match("/CREATE TABLE\s*(\w*)\s*\((.*)\)/", $createStatement, $matches)) {
            $tablename   = $matches[1];
            $definitions = $matches[2];
            $definitions = explode(", ", $definitions);
            foreach ($definitions as $definition) {
                $definition = trim($definition);
                if (preg_match("/^PRIMARY KEY\s*\((.*)\)$/", $definition, $matches)) {
                    $items = explode(",", $matches[1]);
                    foreach ($items as $item) {
                        $dbstructure[$tablename]['primary'][] = $item;
                    }
                } else if (preg_match("/^(KEY|INDEX)\s*(\w*)\s*\((.*)\)$/", $definition, $matches)) {
                    $items = explode(",", $matches[3]);
                    foreach ($items as $item) {
                        $dbstructure[$tablename]['index'][$matches[2]][] = $item;
                    }
                } else if (preg_match("/^UNIQUE\s*(\w*)\s*\((.*)\)$/", $definition, $matches)) {
                    $items = explode(",", $matches[2]);
                    foreach ($items as $item) {
                        $dbstructure[$tablename]['unique'][$matches[1]][] = $item;
                    }
                } else if (preg_match("/^(\w*)\s+(.*)$/", $definition, $matches)) {
                    $dbstructure[$tablename]['columns'][$matches[1]] = $matches[2];
                }
            }
        }
    }
    return $dbstructure;
}

?>