#!/usr/bin/php -q
<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: maintenance.php 1055 2004-11-30 15:52:35Z andrew $
*/

/**
 * @package    MaxMaintenance
 * @subpackage Tools
 * @author     Andrew Hill <andrew@awarez.net>
 *
 * A script file to run to re-generate the AdServer statistics for a given
 * interval, in the event that the raw data tables were incorrect when the 
 * Maintenance Statistics Engine ran.
 */
 
/**
 * The interval start and end dates.
 */
$start  = '2004-12-21 01:00:00';
$end    = '2004-12-21 01:59:59';

// Configure the installation path
define('MAX_PATH', preg_replace('#/maintenance/tools#', '', dirname(__FILE__)));
define('maintenancePath', MAX_PATH . '/maintenance');

// Load the maintenance config file
$conf = parse_ini_file(maintenancePath . '/default.conf.ini', true);
$GLOBALS['_MAX']['CONF'] =& $conf;

// Set the PEAR Date class location
ini_set('include_path', maintenancePath . '/libraries/pear');

// Define the week to start on Sunday (0) so that the Date and Date_Calc
// classes agree on what day is the start of the week
define('DATE_CALC_BEGIN_WEEKDAY', 0);

// Set time limit to 5 minutes and ignore user abort
if (!ini_get('safe_mode')) {
    @set_time_limit(300);
    @ignore_user_abort(true);
}

// Disable magic_quotes_runtime
set_magic_quotes_runtime(0);

// Create Date objects of the start and end dates, and set the "current time"
// to be a second after the end of the operation interval
require_once maintenancePath . '/libraries/pear/Date.php';
$startDate = new Date($start);
$endDate   = new Date($end);
$nowDate   = $endDate;
$nowDate->addSeconds(1);

// Check start/end dates - note that check is the reverse of normal check:
// if the operation interval is <= 60, must be start/end of an hour, to 
// make sure we update all the operation intervals in the hour, and if
// the operation interval > 60, must be the start/end of an operation
// interval, to make sure we update all the hours in the operation interval.
require_once maintenancePath . '/libraries/max/Maintenance.php';
$datesOkay = true;
$operationInterval = $conf['maintenance']['operation_interval'];
if ($operationInterval <= 60) {
    // Must ensure that only one hour is being summarised
    if (!Maintenance::checkDatesInSameHour($startDate, $endDate)) {
        $datesOkay = false;
    }
    // Now check that the start and end dates are match the start and
    // end of the hour
    $hourStart = new Date();
    $hourStart->setYear($startDate->getYear());
    $hourStart->setMonth($startDate->getMonth());
    $hourStart->setDay($startDate->getDay());
    $hourStart->setHour($startDate->getHour());
    $hourStart->setMinute('00');
    $hourStart->setSecond('00');
    $hourEnd = new Date();
    $hourEnd->setYear($endDate->getYear());
    $hourEnd->setMonth($endDate->getMonth());
    $hourEnd->setDay($endDate->getDay());
    $hourEnd->setHour($endDate->getHour());
    $hourEnd->setMinute('59');
    $hourEnd->setSecond('59');
    if (!$startDate->equals($hourStart)) {
        $datesOkay = false;
    }
    if (!$endDate->equals($hourEnd)) {
        $datesOkay = false;
    }
} else {
    // Must ensure that only one operation interval is being summarised
    $operationIntervalID = 
        Maintenance::convertDateRangeToOperationIntervalID($startDate, $endDate, $operationInterval);
    if (is_bool($operationIntervalID) && !$operationIntervalID) {
        $datesOkay = false;
    }
    // Now check that the start and end dates match the start and end
    // of the operation interval
    list($operationIntervalStart, $operationIntervalEnd) =
    Maintenance::convertDateToOperationIntervalStartAndEndDates($startDate, $operationInterval);
    if (!$startDate->equals($operationIntervalStart)) {
        $datesOkay = false;
    }
    if (!$endDate->equals($operationIntervalEnd)) {
        $datesOkay = false;
    }
}
if ($datesOkay == false) {
    echo "\n" . 'Error: The dates set are not valid. See code comments.' . "\n";
    exit();
}

// Give the user a chance to check the dates that will be used
echo 'Regenerating statistics for the range: "' .
     $startDate->format('%Y-%m-%d %H:%M:%S') . '" to "' . 
     $endDate->format('%Y-%m-%d %H:%M:%S') . '"' . "\n";
echo 'Press CTRL-C within 10 seconds to cancel...' . "\n";
sleep(10);

// Create a Data Access Layer object
require_once maintenancePath . '/libraries/max/Dal.php';
$dal = Dal::singleton();

echo 'Restoring incorrect data to campaign limitations...' . "\n";
// Select "bad" data from the data_intermediate_ad table
$query = "
    SELECT
        b.campaignid AS campaign_id,
        c.views AS li,
        c.clicks AS lc,
        c.conversions AS ls,
        SUM(dia.impressions) AS impressions,
        SUM(dia.clicks) AS clicks,
        SUM(dia.conversions) AS conversions
    FROM
        {$conf['table']['banners']} AS b,
        {$conf['table']['campaigns']} AS c,
        {$conf['table']['data_intermediate_ad']} AS dia
    WHERE
        b.campaignid = c.campaignid
        AND b.bannerid = dia.ad_id
        AND dia.interval_start = '" . $startDate->format('%Y-%m-%d %H:%M:%S') . "'
        AND dia.interval_end = '" . $endDate->format('%Y-%m-%d %H:%M:%S') . "'
    GROUP BY
        campaign_id";
$dal->execute($query, $result);
while ($row = mysql_fetch_array($result)) {
    if (($row['li'] >= 0) || ($row['lc'] >= 0) || ($row['ls'] >= 0)) {
        // The campaign has delivery limitation requirements
        if (($row['impressions'] > 0) || ($row['clicks'] > 0) || ($row['conversions'] > 0)) {
            // Restore the campaign delivery limitation requirements for impressions
            if (($row['li'] > 0) && ($row['impressions'] > 0)) {
                // Add the impressions back to the campaign
                $impressions = $row['li'] + $row['impressions'];
                $query = "
                    UPDATE
                        {$conf['table']['campaigns']}
                    SET
                        views = $impressions
                    WHERE
                        campaignid = {$row['campaign_id']}";
                $dal->execute($query, $innerResult);
                echo 'Restored impressions to campiagn ID: ' . $row['campaign_id'] . "\n";
            } else if (($row['li'] == 0) && ($row['impressions'] > 0)) {
                // Add the impressions back to the campaign, and set the
                // campaign to be active again
                $impressions = $row['impressions'];
                $query = "
                    UPDATE
                        {$conf['table']['campaigns']}
                    SET
                        views = $impressions,
                        active = 't'
                    WHERE
                        campaignid = {$row['campaign_id']}";
                $dal->execute($query, $innerResult);
                echo 'Restored impressions to campiagn ID: ' . $row['campaign_id'] . " & set campaign active\n";
            }
            // Restore the campaign delivery limitation requirements for clicks
            if (($row['lc'] > 0) && ($row['clicks'] > 0)) {
                // Add the clicks back to the campaign
                $clicks = $row['lc'] + $row['clicks'];
                $query = "
                    UPDATE
                        {$conf['table']['campaigns']}
                    SET
                        clicks = $clicks
                    WHERE
                        campaignid = {$row['campaign_id']}";
                $dal->execute($query, $innerResult);
                echo 'Restored clicks to campiagn ID: ' . $row['campaign_id'] . "\n";
            } else if (($row['lc'] == 0) && ($row['clicks'] > 0)) {
                // Add the clicks back to the campaign, and set the
                // campaign to be active again
                $clicks = $row['clicks'];
                $query = "
                    UPDATE
                        {$conf['table']['campaigns']}
                    SET
                        clicks = $clicks,
                        active = 't'
                    WHERE
                        campaignid = {$row['campaign_id']}";
                $dal->execute($query, $innerResult);
                echo 'Restored clicks to campiagn ID: ' . $row['campaign_id'] . " & set campaign active\n";
            }
            // Restore the campaign delivery limitation requirements for conversions
            if (($row['ls'] > 0) && ($row['conversions'] > 0)) {
                // Add the conversions back to the campaign
                $conversions = $row['ls'] + $row['conversions'];
                $query = "
                    UPDATE
                        {$conf['table']['campaigns']}
                    SET
                        conversions = $conversions
                    WHERE
                        campaignid = {$row['campaign_id']}";
                $dal->execute($query, $innerResult);
                echo 'Restored conversions to campiagn ID: ' . $row['campaign_id'] . "\n";
            } else if (($row['ls'] == 0) && ($row['conversions'] > 0)) {
                // Add the conversions back to the campaign, and set the
                // campaign to be active again
                $conversions = $row['conversions'];
                $query = "
                    UPDATE
                        {$conf['table']['campaigns']}
                    SET
                        conversions = $conversions,
                        active = 't'
                    WHERE
                        campaignid = {$row['campaign_id']}";
                $dal->execute($query, $innerResult);
                echo 'Restored conversions to campiagn ID: ' . $row['campaign_id'] . " & set campaign active\n";
            }
        }
    }
}

// Delete the previously summarised data
echo 'Deleting summarised data...' . "\n";
// Delete the "bad" data from the data_summary_zone_impression_history table
$query = "
    DELETE FROM
        {$conf['table']['data_summary_zone_impression_history']}
    WHERE
        interval_start >= '" . $startDate->format('%Y-%m-%d %H:%M:%S') . "'
        AND interval_end <= '" . $endDate->format('%Y-%m-%d %H:%M:%S') . "'";
$dal->execute($query, $result);
// Delete the "bad" data from the data_summary_ad_hourly table
$query = "
    DELETE FROM
        {$conf['table']['data_summary_ad_hourly']}
    WHERE
        day >= '" . $startDate->format('%Y-%m-%d') . "'
        AND hour >= " . $startDate->format('%H') . "
        AND day <= '" . $endDate->format('%Y-%m-%d') . "'
        AND hour <= " . $endDate->format('%H');
$dal->execute($query, $result);
// Delete the "bad" data from the data_intermediate_ad_variable_value table
$query = "
    SELECT
        data_intermediate_ad_connection_id
    FROM
        {$conf['table']['data_intermediate_ad_connection']}
    WHERE
        tracker_date_time >= '" . $startDate->format('%Y-%m-%d %H:%M:%S') . "'
        AND tracker_date_time <= '" . $endDate->format('%Y-%m-%d %H:%M:%S') . "'";
$dal->execute($query, $result);
while ($row = mysql_fetch_array($result)) {
    $query = "
        DELETE FROM
            {$conf['table']['data_intermediate_ad_variable_value']}
        WHERE
            data_intermediate_ad_connection_id = {$row['data_intermediate_ad_connection_id']}";
    $dal->execute($query, $innerResult);
}
// Delete the "bad" data from the data_intermediate_ad_connection table
$query = "
    DELETE FROM
        {$conf['table']['data_intermediate_ad_connection']}
    WHERE
        tracker_date_time >= '" . $startDate->format('%Y-%m-%d %H:%M:%S') . "'
        AND tracker_date_time <= '" . $endDate->format('%Y-%m-%d %H:%M:%S') . "'";
$dal->execute($query, $result);
// Delete the "bad" data from the data_intermediate_ad table
$query = "
    DELETE FROM
        {$conf['table']['data_intermediate_ad']}
    WHERE
        interval_start >= '" . $startDate->format('%Y-%m-%d %H:%M:%S') . "'
        AND interval_end <= '" . $endDate->format('%Y-%m-%d %H:%M:%S') . "'";
$dal->execute($query, $result);

// Re-summarise the statistics
require_once maintenancePath . '/libraries/max/Maintenance/Statistics.php';
// Ensure emails are not sent due to activation/deactivation effect
$conf['email']['sendMail'] = false;
// Use fake current time, so that we don't update beyond desired interval,
// and use fake last run date, so that we don't update from before desired
// interval
$lastUpdated = $startDate;
$lastUpdated->subtractSeconds(1);
Maintenance_Statistics::instance('AdServer', $msObj, $nowDate, $lastUpdated);
echo 'Re-generating statistics...' . "\n";
$msObj->summariseIntermediate();
$msObj->summariseFinal();
$msObj->decrementCampaigns();
echo 'Statistics have been re-summarised!' . "\n";

?>