<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: TestOfLib-Log.php 1307 2005-01-27 11:08:45Z andrew $
*/

/**
 * @package    MaxDelivery
 * @subpackage TestSuite
 * @author     Andrew Hill <andrew@awarez.net>
 *
 * @TODO Incomplete - needs DAL sorted out, so that simple creation/dropping
 * of tables in the test database can be achieved.
 */

require_once MAX_PATH . '/libraries/lib-log.inc.php';

/**
 * A class for testing the lib-log.inc.php functions.
 */
class Delivery_TestOfLib_Log extends UnitTestCase
{
    
    /**
     * The constructor method.
     */
    function Delivery_TestOfLib_Log()
    {        
        $this->UnitTestCase();
    }
    
    /**
     * Test the _viewersHostOkayToLog function.
     */
    function test_viewersHostOkayToLog()
    {
        $config =& $GLOBALS['phpAds_config'];
        // Set a fake, known IP address and host name
        $_SERVER['REMOTE_ADDR'] = '24.24.24.24';
        $_SERVER['REMOTE_HOST'] = '';
        // Disable revers lookups
        $config['reverse_lookup'] = false;
        // Set no hosts to ignore
        $config['ignore_hosts'] = array();
        // Test
        $this->assertTrue(_viewersHostOkayToLog());
        // Set different IP addresses to ignore
        $config['ignore_hosts'] = array('23.23.23.23', '127.0.0.1');
        // Test
        $this->assertTrue(_viewersHostOkayToLog());
        // Set different and same IP addresses to ignore
        $config['ignore_hosts'] = array('23.23.23.23', '24.24.24.24', '127.0.0.1');
        // Test
        $this->assertFalse(_viewersHostOkayToLog());
        // Set a fake, known IP address and host name
        $_SERVER['REMOTE_ADDR'] = '24.24.24.24';
        $_SERVER['REMOTE_HOST'] = 'example.com';
        // Set different IP addresses to ignore
        $config['ignore_hosts'] = array('23.23.23.23', 'www.example.com', '127.0.0.1');
        // Test
        $this->assertTrue(_viewersHostOkayToLog());
        // Set different and same IP addresses to ignore
        $config['ignore_hosts'] = array('23.23.23.23', 'example.com', '127.0.0.1');
        // Test
        $this->assertFalse(_viewersHostOkayToLog());
        // Set a fake, known IP address and host name
        $_SERVER['REMOTE_ADDR'] = '127.0.0.1';
        $_SERVER['REMOTE_HOST'] = gethostbyaddr($_SERVER['REMOTE_ADDR']);
        // Enable revers lookups
        $config['reverse_lookup'] = true;
        // Set different IP addresses to ignore
        $config['ignore_hosts'] = array('23.23.23.23', 'example.com');
        // Test
        $this->assertTrue(_viewersHostOkayToLog());
        // Set a fake, known IP address and host name
        $_SERVER['REMOTE_ADDR'] = '127.0.0.1';
        $_SERVER['REMOTE_HOST'] = gethostbyaddr($_SERVER['REMOTE_ADDR']);
        // Set different and same IP addresses to ignore
        $config['ignore_hosts'] = array('23.23.23.23', gethostbyaddr($_SERVER['REMOTE_ADDR']));
        // Test
        $this->assertFalse(_viewersHostOkayToLog());
    }
    
    /**
     * Test the _prepareLogInfo function.
     */
    function test_prepareLogInfo()
    {
        $config =& $GLOBALS['phpAds_config'];
        // Disable reverse lookups
        $config['reverse_lookup'] = false;
        // Disable geotargeting
        $config['geotracking_stats'] = false;
        // Set the remote IP address
        $_SERVER['REMOTE_ADDR'] = '127.0.0.1';
        // Unset the remote host name
        unset($_SERVER['REMOTE_HOST']);
        // Unset the HTTP referer
        unset($_SERVER['HTTP_REFERER']);
        // Unset the user agent
        unset($_SERVER['HTTP_USER_AGENT']);
        // Set a non-SSL port
        $_SERVER['SERVER_PORT'] = 80;
        // Test
        unset($geotargeting);
        unset($zoneInfo);
        unset($userAgentInfo);
        $maxHttps = '';
        _prepareLogInfo(&$geotargeting, &$zoneInfo, &$userAgentInfo, &$maxHttps);
        $this->assertNull($_SERVER['REMOTE_HOST']);
        $this->assertNull($geotargeting);
        $this->assertNull($zoneInfo);
        $this->assertNull($userAgentInfo);
        $this->assertEqual($maxHttps, 0);
        // Enable reverse lookups
        $config['reverse_lookup'] = true;
        // Test
        unset($geotargeting);
        unset($zoneInfo);
        unset($userAgentInfo);
        $maxHttps = '';
        _prepareLogInfo(&$geotargeting, &$zoneInfo, &$userAgentInfo, &$maxHttps);
        $this->assertEqual($_SERVER['REMOTE_HOST'], gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $this->assertNull($geotargeting);
        $this->assertNull($zoneInfo);
        $this->assertNull($userAgentInfo);
        $this->assertEqual($maxHttps, 0);
        // Enable geotargeting
        $config['geotracking_stats'] = true;
        // Set some fake geotargeting info
        $GLOBALS['phpAds_geo']['country_code'] = 'US';
        $GLOBALS['phpAds_geo']['country_name'] = 'United States';
        $GLOBALS['phpAds_geo']['region'] = 'VA';
        $GLOBALS['phpAds_geo']['city'] = 'Herndon';
        $GLOBALS['phpAds_geo']['postal_code'] = '20171';
        $GLOBALS['phpAds_geo']['latitude'] = '38.9252';
        $GLOBALS['phpAds_geo']['longitude'] = '-77.3928';
        $GLOBALS['phpAds_geo']['dma_code'] = '42';
        $GLOBALS['phpAds_geo']['area_code'] = '00';
        $GLOBALS['phpAds_geo']['organisation'] = 'Foo';
        $GLOBALS['phpAds_geo']['isp'] = 'Bar';
        $GLOBALS['phpAds_geo']['netspeed'] = 'Unknown';
        // Test
        unset($geotargeting);
        unset($zoneInfo);
        unset($userAgentInfo);
        $maxHttps = '';
        _prepareLogInfo(&$geotargeting, &$zoneInfo, &$userAgentInfo, &$maxHttps);
        $this->assertEqual($_SERVER['REMOTE_HOST'], gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $this->assertEqual($geotargeting['country_code'], 'US');
        $this->assertEqual($geotargeting['country_name'], 'United States');
        $this->assertEqual($geotargeting['region'], 'VA');
        $this->assertEqual($geotargeting['city'], 'Herndon');
        $this->assertEqual($geotargeting['postal_code'], '20171');
        $this->assertEqual($geotargeting['latitude'], '38.9252');
        $this->assertEqual($geotargeting['longitude'], '-77.3928');
        $this->assertEqual($geotargeting['dma_code'], '42');
        $this->assertEqual($geotargeting['area_code'], '00');
        $this->assertEqual($geotargeting['organisation'], 'Foo');
        $this->assertEqual($geotargeting['isp'], 'Bar');
        $this->assertEqual($geotargeting['netspeed'], 'Unknown');
        $this->assertNull($zoneInfo);
        $this->assertNull($userAgentInfo);
        $this->assertEqual($maxHttps, 0);
        // Set a passed in referer location
        $_GET['loc'] = 'http://www.example.com/test.html';
        // Test
        unset($geotargeting);
        unset($zoneInfo);
        unset($userAgentInfo);
        $maxHttps = '';
        _prepareLogInfo(&$geotargeting, &$zoneInfo, &$userAgentInfo, &$maxHttps);
        $this->assertEqual($_SERVER['REMOTE_HOST'], gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $this->assertEqual($geotargeting['country_code'], 'US');
        $this->assertEqual($geotargeting['country_name'], 'United States');
        $this->assertEqual($geotargeting['region'], 'VA');
        $this->assertEqual($geotargeting['city'], 'Herndon');
        $this->assertEqual($geotargeting['postal_code'], '20171');
        $this->assertEqual($geotargeting['latitude'], '38.9252');
        $this->assertEqual($geotargeting['longitude'], '-77.3928');
        $this->assertEqual($geotargeting['dma_code'], '42');
        $this->assertEqual($geotargeting['area_code'], '00');
        $this->assertEqual($geotargeting['organisation'], 'Foo');
        $this->assertEqual($geotargeting['isp'], 'Bar');
        $this->assertEqual($geotargeting['netspeed'], 'Unknown');
        $this->assertEqual($zoneInfo['scheme'], 0);
        $this->assertEqual($zoneInfo['host'], 'www.example.com');
        $this->assertEqual($zoneInfo['path'], '/test.html');
        $this->assertNull($zoneInfo['query']);
        $this->assertNull($userAgentInfo);
        $this->assertEqual($maxHttps, 0);
        // Set a passed in referer location
        $_GET['loc'] = 'http://www.example.com/test.html';
        // Test
        unset($geotargeting);
        unset($zoneInfo);
        unset($userAgentInfo);
        $maxHttps = '';
        _prepareLogInfo(&$geotargeting, &$zoneInfo, &$userAgentInfo, &$maxHttps);
        $this->assertEqual($_SERVER['REMOTE_HOST'], gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $this->assertEqual($geotargeting['country_code'], 'US');
        $this->assertEqual($geotargeting['country_name'], 'United States');
        $this->assertEqual($geotargeting['region'], 'VA');
        $this->assertEqual($geotargeting['city'], 'Herndon');
        $this->assertEqual($geotargeting['postal_code'], '20171');
        $this->assertEqual($geotargeting['latitude'], '38.9252');
        $this->assertEqual($geotargeting['longitude'], '-77.3928');
        $this->assertEqual($geotargeting['dma_code'], '42');
        $this->assertEqual($geotargeting['area_code'], '00');
        $this->assertEqual($geotargeting['organisation'], 'Foo');
        $this->assertEqual($geotargeting['isp'], 'Bar');
        $this->assertEqual($geotargeting['netspeed'], 'Unknown');
        $this->assertEqual($zoneInfo['scheme'], 0);
        $this->assertEqual($zoneInfo['host'], 'www.example.com');
        $this->assertEqual($zoneInfo['path'], '/test.html');
        $this->assertNull($zoneInfo['query']);
        $this->assertNull($userAgentInfo);
        $this->assertEqual($maxHttps, 0);
        // Set a normal referer        
        $_SERVER['HTTP_REFERER'] = 'https://example.com/test.php?foo=bar';
        // Test
        unset($geotargeting);
        unset($zoneInfo);
        unset($userAgentInfo);
        $maxHttps = '';
        _prepareLogInfo(&$geotargeting, &$zoneInfo, &$userAgentInfo, &$maxHttps);
        $this->assertEqual($_SERVER['REMOTE_HOST'], gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $this->assertEqual($geotargeting['country_code'], 'US');
        $this->assertEqual($geotargeting['country_name'], 'United States');
        $this->assertEqual($geotargeting['region'], 'VA');
        $this->assertEqual($geotargeting['city'], 'Herndon');
        $this->assertEqual($geotargeting['postal_code'], '20171');
        $this->assertEqual($geotargeting['latitude'], '38.9252');
        $this->assertEqual($geotargeting['longitude'], '-77.3928');
        $this->assertEqual($geotargeting['dma_code'], '42');
        $this->assertEqual($geotargeting['area_code'], '00');
        $this->assertEqual($geotargeting['organisation'], 'Foo');
        $this->assertEqual($geotargeting['isp'], 'Bar');
        $this->assertEqual($geotargeting['netspeed'], 'Unknown');
        $this->assertEqual($zoneInfo['scheme'], 1);
        $this->assertEqual($zoneInfo['host'], 'example.com');
        $this->assertEqual($zoneInfo['path'], '/test.php');
        $this->assertEqual($zoneInfo['query'], 'foo=bar');
        $this->assertNull($userAgentInfo);
        $this->assertEqual($maxHttps, 0);
        // Set the user agent
        $_SERVER['HTTP_USER_AGENT'] = 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)';
        // Test
        unset($geotargeting);
        unset($zoneInfo);
        unset($userAgentInfo);
        $maxHttps = '';
        _prepareLogInfo(&$geotargeting, &$zoneInfo, &$userAgentInfo, &$maxHttps);
        $this->assertEqual($_SERVER['REMOTE_HOST'], gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $this->assertEqual($geotargeting['country_code'], 'US');
        $this->assertEqual($geotargeting['country_name'], 'United States');
        $this->assertEqual($geotargeting['region'], 'VA');
        $this->assertEqual($geotargeting['city'], 'Herndon');
        $this->assertEqual($geotargeting['postal_code'], '20171');
        $this->assertEqual($geotargeting['latitude'], '38.9252');
        $this->assertEqual($geotargeting['longitude'], '-77.3928');
        $this->assertEqual($geotargeting['dma_code'], '42');
        $this->assertEqual($geotargeting['area_code'], '00');
        $this->assertEqual($geotargeting['organisation'], 'Foo');
        $this->assertEqual($geotargeting['isp'], 'Bar');
        $this->assertEqual($geotargeting['netspeed'], 'Unknown');
        $this->assertEqual($zoneInfo['scheme'], 1);
        $this->assertEqual($zoneInfo['host'], 'example.com');
        $this->assertEqual($zoneInfo['path'], '/test.php');
        $this->assertEqual($zoneInfo['query'], 'foo=bar');
        $this->assertEqual($userAgentInfo['os'], '2k');
        $this->assertEqual($userAgentInfo['long_name'], 'msie');
        $this->assertEqual($maxHttps, 0);
        // Set the request to be "HTTPS"
        $_SERVER['SERVER_PORT'] = $config['ssl_port'];
        // Test
        unset($geotargeting);
        unset($zoneInfo);
        unset($userAgentInfo);
        $maxHttps = '';
        _prepareLogInfo(&$geotargeting, &$zoneInfo, &$userAgentInfo, &$maxHttps);
        $this->assertEqual($_SERVER['REMOTE_HOST'], gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $this->assertEqual($geotargeting['country_code'], 'US');
        $this->assertEqual($geotargeting['country_name'], 'United States');
        $this->assertEqual($geotargeting['region'], 'VA');
        $this->assertEqual($geotargeting['city'], 'Herndon');
        $this->assertEqual($geotargeting['postal_code'], '20171');
        $this->assertEqual($geotargeting['latitude'], '38.9252');
        $this->assertEqual($geotargeting['longitude'], '-77.3928');
        $this->assertEqual($geotargeting['dma_code'], '42');
        $this->assertEqual($geotargeting['area_code'], '00');
        $this->assertEqual($geotargeting['organisation'], 'Foo');
        $this->assertEqual($geotargeting['isp'], 'Bar');
        $this->assertEqual($geotargeting['netspeed'], 'Unknown');
        $this->assertEqual($zoneInfo['scheme'], 1);
        $this->assertEqual($zoneInfo['host'], 'example.com');
        $this->assertEqual($zoneInfo['path'], '/test.php');
        $this->assertEqual($zoneInfo['query'], 'foo=bar');
        $this->assertEqual($userAgentInfo['os'], '2k');
        $this->assertEqual($userAgentInfo['long_name'], 'msie');
        $this->assertEqual($maxHttps, 1);
    }
    
    /**
     * Test the _insertRawData function.
     */
    function test_insertRawData()
    {
        
    }
    
    /**
     * Test the logAdRequest function.
     */
    function testLogAdRequest()
    {

    }
    
    /**
     * Test the logAdImpression function.
     */
    function testLogAdImpression()
    {

    }
    
    /**
     * Test the logAdClick function.
     */
    function testLogAdClick()
    {

    }
    
    /**
     * Test the logTrackerImpression function.
     */
    function testLogTrackerImpression()
    {

    }
    
    /**
     * Test the logVariableValues function.
     */
    function testLogVariableValues()
    {

    }
    
    /**
     * Test the logTrackerClick function.
     */
    function testLogTrackerClick()
    {

    }
    
    /**
     * Test the logBenchmark function.
     */
    function testLogBenchmark()
    {

    }
    
}

?>
